//-----------------------------------------------------------------------------
//
//	Merit Manager Unit
//
//-----------------------------------------------------------------------------
unit UFilterEnum;

interface

uses
	Windows, ActiveX, DirectShow9, SysUtils, Registry;

const
	IID_IPropertyBag: TGUID = '{55272A00-42CB-11CE-8135-00AA004BB851}';


type
	TFilterItem = record
		Name:	  	WideString;
    FileName:	WideString;
    CLSID:		TGUID;
    Merit:		DWORD;
    Version:	DWORD;
  end;
  PFilterItem = ^TFilterItem;

	TCategoryItem = record
  	Name:		WideString;
    CLSID:	TGUID;
  end;

//-----------------------------------------------------------------------------
//
//	TCategoriesEnum Class
//
//-----------------------------------------------------------------------------
	TCategoriesEnum = class
  private
  	fCategories:	array of TCategoryItem;
    fCount:			Integer;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Enumerate;
    function Count: Integer;
    function GetItem(i: Integer; var Item: TCategoryItem): HRESULT;
  end;

//-----------------------------------------------------------------------------
//
//	TFilterEnum Class
//
//-----------------------------------------------------------------------------
	TFilterEnum = class
  private
  	fFilters:		array of TFilterItem;
    fCount: 		Integer;
    fCategory:		TCategoryItem;
  public
		constructor Create(Category: TCategoryItem);
    destructor Destroy; override;

    procedure Enumerate;
    procedure Clear;

    function Count: Integer;
    function GetItem(i: Integer; var Item: TFilterItem): HRESULT;
  end;

procedure ExtractFilterInfo(var Filter: TFilterItem);
function WriteMerit(var Filter: TFilterItem): BOOL;
function HexToInt(hex: String): Cardinal;

function DeleteFilterFromRegistry(const CLSID: String): Boolean;

function RegisterDLL(FileName: String): HRESULT;
function UnregisterDLL(FileName: String): HRESULT;

implementation

//-----------------------------------------------------------------------------
//
//	TFilterEnum Implementation
//
//-----------------------------------------------------------------------------

function TFilterEnum.GetItem(i: Integer; var Item: TFilterItem): HRESULT;
begin
	// make sure the parameter is valid
	result:= E_FAIL;
  if (i < 0) or (i >= fCount) then exit;
  // return the requested item
  Item := fFilters[i];
  result:= NOERROR;
end;

function TFilterEnum.Count: Integer;
begin
	result:= fCount;
end;

procedure TFilterEnum.Enumerate;
var
  SysDevEnum: ICreateDevEnum;
	EnumCat: IEnumMoniker;
  Moniker: IMoniker;
  f: ULONG;
  hr: HRESULT;
  PropBag: IPropertyBag;
  varName: OleVariant;
  Item: TFilterItem;
begin
	// clear what we may have
  Clear;
  try
  	// make sure everything works fine... we don't want any terrific crashes
  	try
    	// let's create the system device enumerator
     	hr := CoCreateInstance(CLSID_SystemDeviceEnum, NIL, CLSCTX_INPROC_SERVER,
								  IID_ICreateDevEnum, SysDevEnum);
      if FAILED(hr) or (SysDevEnum = NIL) then exit;
      // we want to enumerate categories
			hr := SysDevEnum.CreateClassEnumerator(fCategory.CLSID, EnumCat, 0);
      if FAILED(hr) or (EnumCat = NIL) then exit;
      // scan through the categories
      while (EnumCat.Next(1, Moniker, @F) = NOERROR) do
      begin
      	hr := Moniker.BindToStorage(NIL, NIL, IID_IPropertyBag, PropBag);
        if SUCCEEDED(hr) then
        begin
					// get the category name
          VariantInit(varName);
          PropBag.Read('FriendlyName', varName, NIL);
          Item.Name := WideString(varName);
          VariantClear(varName);
          // get the CLSID
          VariantInit(varName);
          PropBag.Read('CLSID', varName, NIL);
          try
           	Item.CLSID := StringToGuid(WideString(varName));
          except
          	Item.CLSID := GUID_NULL;
          end;
          VariantClear(varName);
          // we're done with property bag
          PropBag := NIL;
          // if the category item is valid, we may add it to our list
          if not IsEqualGUID(Item.CLSID, GUID_NULL) then
          begin
          	ExtractFilterInfo(Item);
           	Inc(fCount);
            SetLength(fFilters, fCount);
            fFilters[fCount-1] := Item;
          end;
        end;
        Moniker:= NIL;
      end;
    except
    	MessageBox(0, 'Unexepcted Error in TFilterEnum.Enumerate', 'Error', MB_ICONERROR);
    end;
  finally
   	// clean all the interfaces
    PropBag:= NIL;
		Moniker:= NIL;
    EnumCat:= NIL;
    SysDevEnum:= NIL;
  end;
end;

procedure TFilterEnum.Clear;
begin
  fCount:= 0;
  SetLength(fFilters, fCount);
end;

constructor TFilterEnum.Create(Category: TCategoryItem);
begin
	inherited Create;

  SetLength(fFilters, fCount);
  fCategory:= Category;

  Enumerate;
end;

destructor TFilterEnum.Destroy;
begin
  Clear;
  inherited Destroy;
end;

//-----------------------------------------------------------------------------
//
//	TCategoriesEnum Implementation
//
//-----------------------------------------------------------------------------

function TCategoriesEnum.GetItem(i: Integer; var Item: TCategoryItem): HRESULT;
begin
	// make sure the parameter is valid
	result := E_FAIL;
  if (i < 0) or (i >= fCount) then exit;
  // return the requested item
  Item := fCategories[i];
  result := NOERROR;
end;

function TCategoriesEnum.Count: Integer;
begin
	result:= fCount;
end;

procedure TCategoriesEnum.Enumerate;
var
  SysDevEnum: ICreateDevEnum;
	EnumCat: IEnumMoniker;
  Moniker: IMoniker;
  f: ULONG;
  hr: HRESULT;
  PropBag: IPropertyBag;
  varName: OleVariant;
  Item: TCategoryItem;
begin
	// clear what we may have
  fCount := 0;
  SetLength(fCategories, 0);
  try
   	// make sure everything works fine... we don't want any terrific crashes
  	try
    	// let's create the system device enumerator
     	hr := CoCreateInstance(CLSID_SystemDeviceEnum, NIL, CLSCTX_INPROC_SERVER,
								  IID_ICreateDevEnum, SysDevEnum);
      if FAILED(hr) or (SysDevEnum = NIL) then exit;
      // we want to enumerate categories
			hr := SysDevEnum.CreateClassEnumerator(CLSID_ActiveMovieCategories, EnumCat, 0);
      if FAILED(hr) or (EnumCat = NIL) then exit;
      // scan through the categories
      while (EnumCat.Next(1, Moniker, @F) = NOERROR) do
      begin
       	hr := Moniker.BindToStorage(NIL, NIL, IID_IPropertyBag, PropBag);
        if SUCCEEDED(hr) then
        begin
					// get the category name
          VariantInit(varName);
          PropBag.Read('FriendlyName', varName, NIL);
          Item.Name := WideString(varName);
          VariantClear(varName);
          // get the CLSID
          VariantInit(varName);
          PropBag.Read('CLSID', varName, NIL);
          try
           	Item.CLSID:= StringToGuid(WideString(varName));
          except
           	Item.CLSID := GUID_NULL;
          end;
          VariantClear(varName);
          // we're done with property bag
          PropBag := NIL;
          // if the category item is valid, we may add it to our list
          if (IsEqualGUID(Item.CLSID, GUID_NULL) = FALSE) then
          begin
          	Inc(fCount);
            SetLength(fCategories, fCount);
            fCategories[fCount-1]:= Item;
          end;
        end;
        Moniker := NIL;
      end;
    except
     	MessageBox(0, 'Unexepcted Error in TCategoriesEnum.Enumerate', 'Error', MB_ICONERROR);
    end;
  finally
  	// clean all the interfaces
    PropBag := NIL;
		Moniker:= NIL;
    EnumCat:= NIL;
    SysDevEnum:= NIL;
  end;
end;

constructor TCategoriesEnum.Create;
begin
	inherited Create;
  fCount:= 0;
  SetLength(fCategories, fCount);
  Enumerate;
end;

destructor TCategoriesEnum.Destroy;
begin
	fCount:= 0;
  SetLength(fCategories, 0);
  inherited Destroy;
end;



//-----------------------------------------------------------------------------
//
//	Globals
//
//-----------------------------------------------------------------------------

procedure ExtractFilterInfo(var Filter: TFilterItem);
var
  Reg: TRegistry;
	s, s2: String;
  Filt: TFilterItem;
  binaryData: record
               	Version: DWORD;
                Merit: DWORD;
                b: array[0..16384] of Byte;
              end;
begin
	Filt := Filter;
  Reg := TRegistry.Create;
  Reg.Access := KEY_READ; //Fixed: only reading is necessary
  try
  	try
    	// file name is located in HKEY_CLASSES_ROOT\CLSID\{filterCLSID}\InprocServer32
      Reg.RootKey:= HKEY_CLASSES_ROOT;
      try
       	s:= GuidToString(Filter.CLSID);
      except
       	s:= GuidToString(GUID_NULL);
      end;
      if (Reg.OpenKey('\CLSID\'+s+'\InprocServer32', FALSE) = TRUE) then
      begin
       	// load the file name
       	Filt.FileName:= Reg.ReadString('');
      end;
      // Merit and other stuff is located in
      // HKEY_CLASSES_ROOT\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\{filterclsid}
      s2:= '\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\';
      //Reg.RootKey:= HKEY_LOCAL_MACHINE;
      //s2:= '\SOFTWARE\Classes\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\';
     	binaryData.Merit := 0;
      binaryData.Version := 0;
      if (Reg.OpenKeyReadOnly(s2 + s) = TRUE) then
      begin
       	try
    	  	Reg.ReadBinaryData('FilterData', binaryData, sizeof(binaryData));
        except
         	binaryData.Merit := 0;
          binaryData.Version := 0;
				end;
      end;
      Filt.Merit := binaryData.Merit;
      Filt.Version := binaryData.Version;
    except
			// error accessing registry
    end;
  finally
  	Reg.Free;
    Filter:= Filt;
  end;
end;

function DeleteFilterFromRegistry(const CLSID: String): Boolean;
var
  Reg: TRegistry;
begin
  result := false;
  if (CLSID = '') or (CLSID[1] <> '{') or (CLSID = '{00000000-0000-0000-0000-000000000000}') then
    exit;
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_CLASSES_ROOT;
  try
    result := Reg.DeleteKey('\CLSID\' + CLSID);
    result := Reg.DeleteKey('\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\' + CLSID) or result;
    result := Reg.DeleteKey('\Filter\' + CLSID) or result;
  except
    result := false;
  end;
  Reg.Free;
end;

function WriteMerit(var Filter: TFilterItem): BOOL;
var
  Reg: TRegistry;
	s, s2: String;
  binaryData: record
               	Version: DWORD;
                Merit: DWORD;
                b: array[0..16384] of Byte;
              end;
  Size: Integer;
begin
  Reg:= TRegistry.Create;
  result := FALSE;
  try
  	try
      Reg.RootKey:= HKEY_CLASSES_ROOT;
      try
      	s := GuidToString(Filter.CLSID);
      except
      	s := GuidToString(GUID_NULL);
      end;
      // Merit and other stuff is located in
      // HKEY_CLASSES_ROOT\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\{filterclsid}
      s2:= '\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\';
      if (Reg.OpenKey(s2 + s, FALSE) = TRUE) then
      begin
       	try
					Size := Reg.ReadBinaryData('FilterData', binaryData, sizeof(binaryData));
          // apply new merit
          binaryData.Merit:= Filter.Merit;
          // write data back
          Reg.WriteBinaryData('FilterData', binaryData, size);
        except
				end;
      end;
      // HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance
      Reg.RootKey := HKEY_LOCAL_MACHINE;
      s2 := '\SOFTWARE\Classes\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\';
      if Reg.OpenKey(s2 + s, FALSE) then
      begin
       	try
					Size := Reg.ReadBinaryData('FilterData', binaryData, sizeof(binaryData));
          // apply new merit
          binaryData.Merit := Filter.Merit;
          // write data back
          Reg.WriteBinaryData('FilterData', binaryData, size);
          result:= TRUE;
        except
				end;
      end;
    except
			// error accessing registry
    end;
  finally
  	Reg.Free;
  end;
end;

function HexToInt(hex: String): Cardinal;
var
  r, r2: Cardinal;
begin
	hex := lowercase(hex);
  r := 0;
  while Length(hex) > 0 do
  begin
   	r := r * 16;
    case (hex[1]) of
      '0'..'9' : r2:= StrToInt(hex[1]);
      'a': r2:= 10;
      'b': r2:= 11;
      'c': r2:= 12;
      'd': r2:= 13;
      'e': r2:= 14;
      'f': r2:= 15;
       else r2:=0; {***}
    end;
    r:= r + r2;
    Delete(hex, 1, 1);
  end;
  result := r;
end;

function RegisterDLL(FileName: String): HRESULT;
var
  RegisterServer: function: HRESULT; stdcall;
	Lib: Cardinal;
begin
	result := E_FAIL;
  Lib := LoadLibrary(PChar(FileName));
  if Lib <> 0 then
  try
    // try to register the ActiveX Server
    RegisterServer:= GetProcAddress(Lib, 'DllRegisterServer');
    if Assigned(RegisterServer) then
      result:= RegisterServer;
  finally
    FreeLibrary(Lib);
  end;
end;

function UnregisterDLL(FileName: String): HRESULT;
var
  UnregisterServer: function: HRESULT; stdcall;
	Lib: Cardinal;
begin
	result := E_FAIL;
  Lib := LoadLibrary(PChar(FileName));
  if Lib <> 0 then
  try
    // try to register the ActiveX Server
    UnregisterServer:= GetProcAddress(Lib, 'DllUnregisterServer');
    if Assigned(UnregisterServer) then
      result:= UnregisterServer;
  finally
    FreeLibrary(Lib);
  end;
end;

initialization
  CoInitialize(NIL);
finalization
	CoUninitialize;
end.
