unit Unit1;

//Changes by Griga - Merit Input Line added. Changes are marked with {***}

//More changes by Griga :-) Log tab & file added. Changes are marked with {+++}

//Yet more changes by Griga - Property Page Button added - marked with {###}

//Again Griga (changes marked with {---}):
//Added button for saving the DirectShow Section as text file.
//Fixed unnecessary registry write access, causing problems in limited user accounts.
//DirectShow filters whose file is missing are displayed in gray.
//In case of an attempt to unregister such a filter the RFM offers to
//delete the according registry keys

interface

{$R WindowsXP.res}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  StdCtrls, ComCtrls, Menus, ExtCtrls, Dialogs, UFilterEnum, ActiveX, DirectShow9,
  ShellAPI, ClipBrd;

type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    BClose: TButton;
    PopupMenu1: TPopupMenu;
    ItemUnregister: TMenuItem;
    N1: TMenuItem;
    TabSheet2: TTabSheet;
    Label1: TLabel;
    TabSheet3: TTabSheet;
    InfoMemo: TMemo;
    Label2: TLabel;
    Image1: TImage;
    Label3: TLabel;
    Image2: TImage;
    BRegister: TButton;
    BUnregister: TButton;
    CBMerit: TComboBox;
    BSetValue: TButton;
    OpenD: TOpenDialog;
    ItemCopyCLSID: TMenuItem;
    TVFilters: TTreeView;
    BCopyGUID: TButton;
    BNavigate: TButton;
    Expand1: TMenuItem;
    Collapse1: TMenuItem;
    EMerit: TEdit;
    TabSheet4: TTabSheet;
    MLog: TMemo;
    CBLog: TCheckBox;
    BClearLog: TButton;
    BPropertyPage: TButton;
    Label4: TLabel;
    BDShowLog: TButton;
    SaveD: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TVFiltersCustomDrawItem(Sender: TCustomTreeView;
      Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure BCloseClick(Sender: TObject);
    procedure Expand1Click(Sender: TObject);
    procedure Collapse1Click(Sender: TObject);
    procedure TVFiltersChange(Sender: TObject; Node: TTreeNode);
    procedure BSetValueClick(Sender: TObject);
    procedure BNavigateClick(Sender: TObject);
    procedure BCopyGUIDClick(Sender: TObject);
    procedure BRegisterClick(Sender: TObject);
    procedure BUnregisterClick(Sender: TObject);
    procedure EKeyPress(Sender: TObject; var Key: Char);
    procedure CBMeritOnChange(Sender: TObject);
    procedure BClearClick(Sender: TObject);
    procedure BPropertyPageClick(Sender: TObject);
    procedure QueryEndMessage(var Message: TMessage); Message WM_QUERYENDSESSION;
    procedure BDShowLogClick(Sender: TObject);
  private
    { Private declarations }
    LogPath: String; {+++}
    procedure Log(const s: String); {+++}
    procedure LogEnumFilters(const Action: String; Filename: String); {+++}
    function MyMessageBox(const Text: String; uType: DWord): Boolean;
  public
    { Public declarations }
    Categories: TCategoriesEnum;
    CurrentFilter: TTreeNode;

    procedure BuildTree;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

const
  MeritPreText = 'Merit : 0x'; {***}
  CLSIDPreText = 'CLSID : ';   {***}
  FilePreText = 'File Name : '; {+++}
  VersionPreText = 'Version : '; {+++}
  LogName = 'FilterManager.log'; {+++}

function TMainForm.MyMessageBox(const Text: String; uType: DWord): Boolean;
begin
  result := MessageBox(Handle,PChar(Text),'RadLight Filter Manager',uType) = IDOK;
end;

procedure TMainForm.Log(const s: String); {+++}
begin
  with MLog.Lines do
  begin
    if Count = 0 then
      Add('*** RadLight Filter Manager Log File ***');
    Add('');
    Add(DateTimeToStr(Now));
    Add(s);
  end;
end;

procedure TMainForm.LogEnumFilters(const Action: String; Filename: String); {+++}
// Log all filters that are contained in the file
var
  i: Integer;
begin
  Log(Action + Filename);
  Filename := LowerCase(Filename);
  for i := 0 to TVFilters.Items.Count-1 do
    with TVFilters.Items[i] do
      if (Level = 1) and
         (Filename = LowerCase(copy(Item[1].Text,Length(FilePreText)+1,Length(Filename)))) then
        MLog.Lines.Add(Text);
end;

procedure TMainForm.BRegisterClick(Sender: TObject);
var
  hr: HRESULT;
	i: Integer;
  s: String;
begin
	if OpenD.Execute then
  begin
  	for i := 0 to OpenD.Files.Count-1 do
    begin
			hr := RegisterDLL(OpenD.Files.Strings[i]);
      if SUCCEEDED(hr) then
      begin
        // refresh the filter tree
        BuildTree;
        LogEnumFilters('Registered: ',OpenD.Files.Strings[i]);  {+++}
       	s := 'DllRegisterServer succeeded in : '+OpenD.Files.Strings[i];
				MyMessageBox(s, MB_ICONINFORMATION);
      end
      else begin
       	s := 'DllRegisterServer failed in : '+OpenD.Files.Strings[i]+' with error code : 0x';
        s := s + IntToHex(hr, 8);
				MyMessageBox(s, MB_ICONERROR);
      end;
    end;
  end;
end;

procedure TMainForm.BUnregisterClick(Sender: TObject);
var
  s,s2: String;
	hr: HRESULT;
begin
	if CurrentFilter = NIL then exit;
  try
   	s := CurrentFilter.Item[1].Text;
    Delete(s, 1, Length(FilePreText));
    hr := UnregisterDLL(s);
    if SUCCEEDED(hr) then
    begin
      LogEnumFilters('Unregistered: ',s); {+++}
			MyMessageBox('DllUnregisterServer succeeded in : ' + s, MB_ICONINFORMATION);
    end
    {---}
    else if (CurrentFilter.StateIndex = -1) and not FileExists(s) then //file does not exist
    begin
      if MyMessageBox('Cannot unregister - file is missing. Delete the registry entries?',
                      MB_OKCANCEL or MB_ICONQUESTION) then
      begin
        hr := S_OK;
        s2 := CurrentFilter.Item[0].Text; //CLSID
        Delete(s2, 1, Length(CLSIDPreText));
        if DeleteFilterFromRegistry(s2) then
          LogEnumFilters('Removed registry keys: ',s)
        else begin
          MyMessageBox('Deleting the registry entries failed', MB_ICONERROR);
          hr := E_FAIL;
        end;
      end
    end
    else begin
    	s := 'DllUnregisterServer failed in : '+ s +' with error code : 0x' + IntToHex(hr, 8);
      MyMessageBox(s, MB_ICONERROR);
    end;
  except
    hr := E_FAIL;
  end;
  // refresh the filter tree
  if SUCCEEDED(hr) then
    BuildTree;
end;

procedure TMainForm.BNavigateClick(Sender: TObject);
var
  s: String;
begin
	if CurrentFilter = NIL then exit;
	// open the explorer in the filter directory
  try
  	s := CurrentFilter.Item[1].Text;
    Delete(s, 1, Length(FilePreText));
    s := ExtractFilePath(s);
    ShellExecute(Handle, 'open', PChar(s), '', '', SW_SHOWNORMAL);
  except
  end;
end;

procedure TMainForm.BCopyGUIDClick(Sender: TObject);
var
  s: String;
begin
	if CurrentFilter = NIL then exit;
	// put the required CLSID into clipboard
  try
    s := CurrentFilter.Item[0].Text;
    Delete(s, 1, Length(CLSIDPreText)); {***}
    ClipBoard.SetTextBuf(PChar(s));
  except
  end;
end;

procedure TMainForm.BSetValueClick(Sender: TObject);
var
  Item: TFilterItem;
	s: String;
  NewMerit: Cardinal;
begin
	if CurrentFilter = NIL then exit;
	// we don't have the reference to the filter item anymore... so let's build one
  Item.Name:= CurrentFilter.Text;
  // subitems are as follows :
  //  1. CLSID
  //  2. File Name (not necesary)
  //	3. Merit
  //	4. Version  (not necesary)
  try
  	// clsid
    s := CurrentFilter.Item[0].Text;
    Delete(s, 1, Length(CLSIDPreText)); {***}
    try
    	Item.CLSID:= StringToGuid(s);
   except
    Item.CLSID:= GUID_NULL;
   end;
   // filename
   s := CurrentFilter.Item[1].Text;
   Item.FileName := s;
   // merit
   s := CurrentFilter.Item[2].Text;
   Delete(s, 1, Length(MeritPreText)); {***}
   Item.Merit := HexToInt(s);
   // set the new merit  {***}
   NewMerit := HexToInt(EMerit.Text);
   Item.Merit := NewMerit;
   if WriteMerit(Item) then
   begin
     Log('Changed merit: ' + CurrentFilter.Text); {+++}
     MLog.Lines.Add(s + ' --> ' + EMerit.Text);
     CurrentFilter.Item[2].Text := MeritPreText+IntToHex(NewMerit, 8);
   end
   else
     MyMessageBox('Cannot update merit', MB_ICONERROR);
  except
  	// we don't care..
  end;
end;

{###}
procedure TMainForm.BPropertyPageClick(Sender: TObject);
const
  IID_ISpecifyPropertyPages: TGUID = '{B196B28B-BAB4-101A-B69C-00AA00341D07}';
var
  s: String;
  CLSID: TGuid;
  Filter: IBaseFilter;
  pSpecify: ISpecifyPropertyPages;
  caGuid: tagCAUUID;
begin
	if CurrentFilter = NIL then exit;
  try
    s:= CurrentFilter.Item[0].Text;
    Delete(s, 1, Length(CLSIDPreText));
    CLSID:= StringToGuid(s);
    if SUCCEEDED(CoCreateInstance(CLSID,nil,CLSCTX_INPROC_SERVER,
           IID_IBaseFilter,Filter)) and
       SUCCEEDED(Filter.QueryInterface(IID_ISpecifyPropertyPages,
           pSpecify)) then
    begin
      pSpecify.GetPages(caGuid);
      if caGuid.cElems > 0 then
      begin
        OleCreatePropertyFrame(
         Handle, // Parent window
         0, // x (Reserved)
         0, // y (Reserved)
         PWideChar(WideString(CurrentFilter.Text)), // Caption for the dialog box
         1, // Number of filters
         @Filter, // Pointer to the filter
         caGUID.cElems, // Number of property pages
         caGUID.pElems, // Pointer to property page CLSIDs
         0, // Locale identifier
         0, // Reserved
         nil // Reserved
        );
        CoTaskMemFree(caGUID.pElems);
      end;
    end;
  except
  end;
end;

procedure TMainForm.BuildTree;
var
  Node, SubNode: TTreeNode;
	i, i2: Integer;
  Cat: TCategoryItem;
  Filt: TFilterItem;
  Filters: TFilterEnum;
  s: String;
  IsDirectShow: Boolean;
begin
  TVFilters.Items.BeginUpdate;
  try
    TVFilters.Items.Clear;
    TVFilters.SortType:= stNone;
    // scan through the categories
    for i := 0 to Categories.Count -1 do
    begin
      if FAILED(Categories.GetItem(i, Cat)) then continue;
      Node:= TVFilters.Items.Add(NIL, Cat.Name);
      IsDirectShow := Pos('directshow',LowerCase(Node.Text)) > 0;
      Filters:= TFilterEnum.Create(Cat);
      try
        for i2:= 0 to Filters.Count -1 do
        begin
          if FAILED(Filters.GetItem(i2, Filt)) then continue;
          SubNode:= TVFilters.Items.AddChild(Node, Filt.Name);
          // add filter information :
          // FileName
          TVFilters.Items.AddChild(SubNode, FilePreText+Filt.FileName);
          if not IsDirectShow or FileExists(Filt.FileName) then
            SubNode.StateIndex := 0
          else
            SubNode.StateIndex := -1;
          // GUID
          try
            s := GuidToString(Filt.CLSID);
          except
            s := 'INVALID GUID';
          end;
          TVFilters.Items.AddChild(SubNode, CLSIDPreText+s); {***}
          // Merit
          TVFilters.Items.AddChild(SubNode, MeritPreText+IntToHex(Filt.Merit, 8));
          // Version
          TVFilters.Items.AddChild(SubNode, VersionPreText+IntToStr(Filt.Version));
        end;
      finally
        Filters.Free;
      end;
    end;
    TVFilters.SortType:= stText;
	finally
    TVFilters.Items.EndUpdate;
  end;
end;

procedure TMainForm.TVFiltersCustomDrawItem(Sender: TCustomTreeView;
  Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
begin
	with TVFilters.Canvas do
  begin
    case Node.Level of
      0: // drawing a category
        Font.Color:= clBlack; 
      1: // drawing a filter
        if Node.StateIndex = 0 then //file exists
          Font.Color:= clNavy
        else
          Font.Color := clGray;
      2: // drawing a information
        if (Node.Index <> 1) or (Node.Parent.StateIndex = 0) then //file exists
          Font.Color:= clRed
        else
          Font.Color := clGray;
    end;
    // selected color
    if cdsSelected in State then
    begin
 			Brush.Color:= RGB(200, 220, 255);
      Pen.Color:= RGB(200, 220, 255);
    end;
  end;
end;

procedure TMainForm.TVFiltersChange(Sender: TObject; Node: TTreeNode);
begin
 	CurrentFilter:= NIL;
  try
    if Node = NIL then exit;
    // we have a category selected... - no filter
 		if Node.Level = 0 then exit;
		// Filters are Level 1
    if Node.Level = 1 then
    begin
    	CurrentFilter:= Node;
      exit;
    end;
    // we must have an information node selected -> find its parent
    CurrentFilter:= Node.Parent;
  finally
  	// enable / disable unregister button
   	BUnregister.Enabled:= CurrentFilter <> NIL;
    ItemUnregister.Enabled:= BUnregister.Enabled;
    ItemCopyCLSID.Enabled:= BUnregister.Enabled;
    BNavigate.Enabled:= BUnregister.Enabled;
    BCopyGUID.Enabled:= BUnregister.Enabled;
    {***}
    BSetValue.Enabled:=BUnregister.Enabled;
    EMerit.Enabled:=BUnregister.Enabled;
    {###}
    BPropertyPage.Enabled:=BUnregister.Enabled;
    //Transfer the merit to the input line, if it is selected
    if EMerit.Enabled {and (pos(MeritPreText,Node.Text) = 1)} then
      EMerit.Text:=Copy(CurrentFilter.Item[2].Text,Length(MeritPreText)+1,8);
  end;
end;

procedure TMainForm.Expand1Click(Sender: TObject);
begin
	TVFilters.FullExpand;
end;

procedure TMainForm.Collapse1Click(Sender: TObject);
begin
	TVFilters.FullCollapse;
end;

procedure TMainForm.EKeyPress(Sender: TObject; var Key: Char); {***}
//keys for hex input
begin
  if not (Key in [#1..#31,'0'..'9','A'..'F','a'..'f']) then key := #0;
end;

procedure TMainForm.CBMeritOnChange(Sender: TObject); {***}
var
  NewMerit: Cardinal;
begin
  // set the new merit
	case (CBMerit.ItemIndex) of
  	0: NewMerit:= MERIT_PREFERRED + 255;
    1: NewMerit:= MERIT_PREFERRED + 1;
    2: NewMerit:= MERIT_PREFERRED;
    3: NewMerit:= MERIT_NORMAL + 1;
    4: NewMerit:= MERIT_NORMAL;
    5: NewMerit:= MERIT_UNLIKELY;
    6: NewMerit:= MERIT_DO_NOT_USE;
    7: NewMerit:= MERIT_SW_COMPRESSOR;
    8: NewMerit:= MERIT_HW_COMPRESSOR;
    else NewMerit:= MERIT_UNLIKELY;	// in case something went wrong.. you never know..
  end;
  EMerit.Text:=IntToHex(NewMerit, 8);
end;

procedure TMainForm.BClearClick(Sender: TObject);   {+++}
begin
  MLog.Lines.Clear;
end;

//-----------------------------------------------------------------------------
//
//	Form Initializing & Uninitializing
//
//-----------------------------------------------------------------------------

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LogPath := ExtractFilePath(ParamStr(0))+LogName; {+++}
  CBLog.Checked := FileExists(LogPath);
  if CBLog.Checked then
  try
    MLog.Lines.LoadFromFile(LogPath);
  except
  end;
	Categories:= TCategoriesEnum.Create;
  BuildTree;
end;

procedure TMainForm.QueryEndMessage(var Message: TMessage);
begin
  try {+++}
    if CBLog.Checked then
      MLog.Lines.SaveToFile(LogPath)
    else
      DeleteFile(LogPath);
  except
  end;
  Message.Result := 1;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  try {+++}
    if CBLog.checked then
      MLog.Lines.SaveToFile(LogPath)
    else
      DeleteFile(LogPath);
  except
  end;
	Categories.Free;
end;

procedure TMainForm.BCloseClick(Sender: TObject);
begin
	Close;
end;

procedure TMainForm.BDShowLogClick(Sender: TObject); {---}
const
  Delimiter = ','; //comma separated
var
  Filename, Merit, CLSID: String;
  List: TStringList;
  i: Integer;
  Node, SubNode: TTreenode;
begin
  Node := TVFilters.Items[0];
  //find the DirectShow Section
  while Node <> nil do
  begin
    if Pos('directshow',LowerCase(Node.Text)) > 0 then
      break;
    Node := Node.GetNextSibling;
  end;
  if Node <> nil then
  begin
    if not SaveD.Execute then exit;
    //Now list all DS filters
    List := TStringList.Create;
    try
      List.Add('Filter' + Delimiter +
               'CLSID' + Delimiter +
               'Merit' + Delimiter +
               'Path');
      for i := 0 to Node.Count - 1 do
      begin
        Filename := ''; Merit := ''; CLSID := '';
        SubNode := Node.Item[i];
        if SubNode.Count > 0 then
          CLSID := Copy(SubNode.Item[0].Text,Length(CLSIDPreText)+1,255);
        if SubNode.Count > 1 then
        begin
          Filename := Copy(SubNode.Item[1].Text,Length(FilePreText)+1,255);
          if SubNode.StateIndex = -1 then
            Filename := '!_' + Filename;
        end;
        if SubNode.Count > 2 then
          Merit := Copy(SubNode.Item[2].Text,Length(MeritPreText)+1,255);
        List.Add('"' + SubNode.Text + '"' + Delimiter + CLSID + Delimiter +
                 Merit + Delimiter + '"' + Filename + '"');
      end;
      List.SaveToFile(SaveD.FileName);
    except
    end;
    List.Free;
  end;
end;

end.
